// --------------------------------------------------------------------
const usageData = {
  tc__translateButton: false,
  tc__openLinkButton: false,
  tc__wiktionaryButton: true,
  tc__geizhalsButton: true,
  tc__googleSelectionButton: true
}

// --------------------------------------------------------------------
function loadValues (data) {
  for (const key of Object.keys(data)) {
    if (usageData[key] !== undefined) {
      usageData[key] = data[key]
    }
  }

  updateUI()
}

// --------------------------------------------------------------------
function updateUI () {
  for (const key of Object.keys(usageData)) {
    const uiTarget = document.getElementById(key)
    if (uiTarget !== null) {
      if (usageData[key] === true) {
        if (uiTarget.getAttribute('type') === 'checkbox') {
          uiTarget.setAttribute('checked', 'checked')
        }
      } else {
        if (uiTarget.getAttribute('type') === 'checkbox') {
          uiTarget.removeAttribute('checked')
        }
      }
    }
  }
}

// --------------------------------------------------------------------
function saveValues (data) {
  browser.storage.local.set(data).then(function () {
    console.log('TextCompanion: Data saved.')
    updateUI()
  }, function (err) {
    console.log('TextCompanion: Error saving data.')
    console.log(err)
    updateUI()
  })
}

// --------------------------------------------------------------------
function saveData (evt) {
  evt.preventDefault()

  const target = evt.target
  const key = target.getAttribute('id')

  if (target.getAttribute('type') === 'checkbox') {
    if (target.getAttribute('checked') === null) {
      target.setAttribute('checked', 'checked')
      usageData[key] = true
    } else {
      target.removeAttribute('checked')
      usageData[key] = false
    }
  }
  saveValues(usageData)
}

// --------------------------------------------------------------------
function onLoadError (err) {
  console.log('TextCompanion: Error while loading internal data.')
  console.log(err)
}

// --------------------------------------------------------------------
const buttonSettings = document.querySelectorAll('#buttonSettings .buttonOption')
for (const item of buttonSettings) {
  item.addEventListener('change', saveData)
}

// --------------------------------------------------------------------------------
browser.storage.local.get().then(loadValues, onLoadError)
